==> 2.0.4.tar.gz: lib/src/firework_sdk.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';

import 'constants/fw_error_name.dart';
import 'constants/fw_event_name.dart';
import 'fw_event_bus.dart';
import 'fw_navigator.dart';
import 'live_stream.dart';
import 'models/ad_badge_configuration.dart';
import 'models/fw_error.dart';
import 'models/fw_events.dart';
import 'models/method_names.dart';
import 'models/track_purchase_parameters.dart';
import 'models/video_launch_behavior.dart';
import 'models/video_player_configuration.dart';
import 'utils/fw_global_state.dart';
import 'utils/fw_method_channel_util.dart';
import 'utils/model_utils/ad_badge_configuration_util.dart';
import 'utils/model_utils/fw_events_util.dart';
import 'utils/model_utils/method_names_util.dart';
import 'utils/model_utils/track_purchase_parameters_util.dart';
import 'utils/model_utils/video_player_configuration_util.dart';
import 'video_shopping.dart';

typedef SDKInitCallback = FutureOr<void> Function(SDKInitEvent event);
typedef CustomCTAClickCallback = FutureOr<void> Function(
    CustomCTAClickEvent? event);
typedef VideoPlaybackCallback = FutureOr<void> Function(
    VideoPlaybackEvent? event);
typedef VideoFeedClickCallback = FutureOr<void> Function(
    VideoFeedClickEvent? event);

/// The entry class of Firework Flutter SDK, which supports the sdk initialization, global configuration, etc.
class FireworkSDK {
  static FireworkSDK? _instance;

  /// The callback of SDK initialization.
  SDKInitCallback? onSDKInit;

  /// The callback of clicking video feed.
  VideoFeedClickCallback? onVideoFeedClick;

  /// The callback of clicking custom CTA.
  CustomCTAClickCallback? get onCustomCTAClick => _onCustomCTAClick;
  set onCustomCTAClick(CustomCTAClickCallback? newValue) {
    _onCustomCTAClick = newValue;

    final nativeMethodName =
        FireworkSDKNativeMethodName.setCustomCTAClickEnabled.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, newValue != null);
  }

  CustomCTAClickCallback? _onCustomCTAClick;

  /// The callback of video playback.
  VideoPlaybackCallback? get onVideoPlayback => _onVideoPlayback;

  set onVideoPlayback(VideoPlaybackCallback? newValue) {
    _onVideoPlayback = newValue;

    final nativeMethodName =
        FireworkSDKNativeMethodName.setVideoPlaybackEventEnabled.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeListMethod(nativeMethodName, newValue != null);
  }

  VideoPlaybackCallback? _onVideoPlayback;

  /// The share base URL of videos.
  String? get shareBaseURL => _shareBaseURL;

  set shareBaseURL(String? shareBaseURL) {
    final shareBaseURLChanged = _shareBaseURL != shareBaseURL;
    _shareBaseURL = shareBaseURL;
    if (shareBaseURLChanged) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.shareBaseURLUpdated,
        ),
      );
    }

    final nativeMethodName = FireworkSDKNativeMethodName.setShareBaseURL.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeListMethod(nativeMethodName, _shareBaseURL);
  }

  String? _shareBaseURL;

  /// The configuration for ad badges.
  AdBadgeConfiguration? get adBadgeConfiguration => _adBadgeConfiguration;
  set adBadgeConfiguration(AdBadgeConfiguration? adBadgeConfiguration) {
    final valueHasChanged = _adBadgeConfiguration?.badgeTextType !=
            adBadgeConfiguration?.badgeTextType ||
        _adBadgeConfiguration?.backgroundColor !=
            adBadgeConfiguration?.backgroundColor ||
        _adBadgeConfiguration?.textColor != adBadgeConfiguration?.textColor;

    _adBadgeConfiguration = adBadgeConfiguration;
    if (valueHasChanged) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.adBadgeConfigurationUpdated,
        ),
      );
    }

    final nativeMethodName =
        FireworkSDKNativeMethodName.setAdBadgeConfiguration.name;
    Map<String, dynamic> arg = {};
    if (adBadgeConfiguration != null) {
      arg = AdBadgeConfigurationUtil.toJson(adBadgeConfiguration);
    }
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg);
  }

  AdBadgeConfiguration? _adBadgeConfiguration;

  /// Get video launch behavior.
  VideoLaunchBehavior? get videoLaunchBehavior => _videoLaunchBehavior;
  VideoLaunchBehavior? _videoLaunchBehavior;

  /// Get current app language.
  String? get appLanguage => _appLanguage;
  String? _appLanguage;

  static FireworkSDK _getInstance() {
    _instance ??= FireworkSDK._();
    return _instance!;
  }

  /// Get global single instance of FireworkSDK class.
  factory FireworkSDK.getInstance() => _getInstance();

  FireworkSDK._() {
    FWMethodChannelUtil.getFirewokSDKChannel()
        .setMethodCallHandler((call) async {
      final fwMethod = FireworkSDKMethodNameUtil.createMethodName(call.method);
      if (fwMethod == null) {
        return null;
      }

      switch (fwMethod) {
        case FireworkSDKMethodName.sdkInit:
          final sdkInitEvent = SDKInitEventUtil.fromJson(call.arguments);
          if (sdkInitEvent != null) {
            onSDKInit?.call(
              sdkInitEvent,
            );
          } else {
            onSDKInit?.call(
              SDKInitEvent(
                error: FWError(
                  name: FWErrorName.parseJsonError,
                  reason: "Fail to parse SDK init event",
                ),
              ),
            );
          }
          break;
        case FireworkSDKMethodName.customCTAClick:
          onCustomCTAClick?.call(
            CustomCTAClickEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.videoPlayback:
          onVideoPlayback?.call(
            VideoPlaybackEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.videoFeedClick:
          onVideoFeedClick?.call(
            VideoFeedClickEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.nativeAppLanguageUpdated:
          final arg = call.arguments;
          if (arg is Map<dynamic, dynamic>) {
            if (arg["appLanguage"] is String) {
              _updateInternalAppLanguage(arg["appLanguage"] as String);
            }
          }
          break;
      }

      return null;
    });
  }

  /// LiveStream instance.
  LiveStream get liveStream => LiveStream.getInstance();

  /// VideoShopping instance.
  VideoShopping get shopping => VideoShopping.getInstance();

  /// FWNavigator instance.
  FWNavigator get navigator => FWNavigator.getInstance();

  /// Initializes Firework SDK.
  ///
  /// [userId] is used to uniquely identify device or user.
  Future<void> init({
    String? userId,
    VideoLaunchBehavior? videoLaunchBehavior,
  }) async {
    final videoLaunchBehaviorUpdated =
        _videoLaunchBehavior != videoLaunchBehavior;
    if (videoLaunchBehaviorUpdated) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.videoLaunchBehaviorUpdated,
        ),
      );
    }

    final nativeMethodName = FireworkSDKNativeMethodName.init.name;
    Map<String, dynamic> arg = {};
    if (userId != null) {
      arg["userId"] = userId;
    }

    if (videoLaunchBehavior != null) {
      arg["videoLaunchBehavior"] = videoLaunchBehavior.name;
    }

    await FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg);

    final shoppingInitNativeMethodName = ShoppingNativeMethodName.init.name;
    await FWMethodChannelUtil.getShoppingChannel().invokeMethod(
      shoppingInitNativeMethodName,
    );

    final liveStreamInitNativeMethodName = LiveStreamNativeMethodName.init.name;
    await FWMethodChannelUtil.getLiveStreamMethodChannel()
        .invokeMethod(liveStreamInitNativeMethodName);

    FWGlobalState.getInstance().completeSdkInitCalledFuture();
  }

  /// Opens video [url] with the [config].
  Future<void> openVideoPlayer({
    required String url,
    VideoPlayerConfiguration? config,
  }) async {
    if (!FWGlobalState.getInstance().sdkInitCalled) {
      await FWGlobalState.getInstance().sdkInitCalledFuture;
    }
    final nativeMethodName = FireworkSDKNativeMethodName.openVideoPlayer.name;
    Map<String, dynamic> arg = {
      "url": url,
    };

    if (config != null) {
      arg["config"] = VideoPlayerConfigurationUtil.toJson(config);
    }

    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg);
  }

  /// Records the user purchase.
  Future<void> trackPurchase(TrackPurchaseParameters parameters) async {
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      if (!FWGlobalState.getInstance().sdkInitCalled) {
        await FWGlobalState.getInstance().sdkInitCalledFuture;
      }
      final nativeMethodName = FireworkSDKNativeMethodName.trackPurchase.name;
      FWMethodChannelUtil.getFirewokSDKChannel().invokeMethod(
          nativeMethodName, TrackPurchaseParametersUtil.toJson(parameters));
    }
  }

  /// Change App level language.
  /// language can be en, ar and en-US.
  /// If language is null or empty string, SDK will use system language.
  Future<bool> changeAppLanguage(String? language) async {
    if (!FWGlobalState.getInstance().sdkInitCalled) {
      await FWGlobalState.getInstance().sdkInitCalledFuture;
    }

    final nativeMethodName = FireworkSDKNativeMethodName.changeAppLanguage.name;
    final result = await FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, language);
    if (result) {
      if (defaultTargetPlatform == TargetPlatform.iOS) {
        await navigator.stopFloatingPlayer();
      }
      _updateInternalAppLanguage(language);
    }
    return result;
  }

  void _updateInternalAppLanguage(String? language) {
    final valueHasChanged = language != _appLanguage;
    _appLanguage = language;
    if (valueHasChanged) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.appLanguageUpdated,
        ),
      );
    }
  }
}

==> 2.4.0.tar.gz: lib/src/firework_sdk.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';

import 'constants/fw_error_name.dart';
import 'constants/fw_event_name.dart';
import 'fw_event_bus.dart';
import 'fw_navigator.dart';
import 'live_stream.dart';
import 'models/ad_badge_configuration.dart';
import 'models/fw_error.dart';
import 'models/fw_events.dart';
import 'models/method_names.dart';
import 'models/open_video_player_configuration.dart';
import 'models/track_purchase_parameters.dart';
import 'models/video_launch_behavior.dart';
import 'utils/fw_global_state.dart';
import 'utils/fw_logger_util.dart';
import 'utils/fw_method_channel_util.dart';
import 'utils/model_utils/ad_badge_configuration_util.dart';
import 'utils/model_utils/fw_events_util.dart';
import 'utils/model_utils/method_names_util.dart';
import 'utils/model_utils/track_purchase_parameters_util.dart';
import 'utils/model_utils/video_player_configuration_util.dart';
import 'video_shopping.dart';

typedef SDKInitCallback = FutureOr<void> Function(SDKInitEvent event);
typedef CustomCTAClickCallback = FutureOr<void> Function(
    CustomCTAClickEvent? event);
typedef VideoPlaybackCallback = FutureOr<void> Function(
    VideoPlaybackEvent? event);
typedef VideoFeedClickCallback = FutureOr<void> Function(
    VideoFeedClickEvent? event);

/// The entry class of Firework Flutter SDK, which supports the sdk initialization, global configuration, etc.
class FireworkSDK {
  static FireworkSDK? _instance;

  /// The callback of SDK initialization.
  SDKInitCallback? onSDKInit;

  /// The callback of clicking video feed.
  VideoFeedClickCallback? onVideoFeedClick;

  /// The callback of clicking custom CTA.
  CustomCTAClickCallback? get onCustomCTAClick => _onCustomCTAClick;
  set onCustomCTAClick(CustomCTAClickCallback? newValue) {
    _onCustomCTAClick = newValue;

    final nativeMethodName =
        FireworkSDKNativeMethodName.setCustomCTAClickEnabled.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, newValue != null);
  }

  CustomCTAClickCallback? _onCustomCTAClick;

  /// The callback of video playback.
  VideoPlaybackCallback? get onVideoPlayback => _onVideoPlayback;

  set onVideoPlayback(VideoPlaybackCallback? newValue) {
    _onVideoPlayback = newValue;

    final nativeMethodName =
        FireworkSDKNativeMethodName.setVideoPlaybackEventEnabled.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, newValue != null);
  }

  VideoPlaybackCallback? _onVideoPlayback;

  /// The share base URL of videos.
  String? get shareBaseURL => _shareBaseURL;

  set shareBaseURL(String? shareBaseURL) {
    final shareBaseURLChanged = _shareBaseURL != shareBaseURL;
    _shareBaseURL = shareBaseURL;

    final nativeMethodName = FireworkSDKNativeMethodName.setShareBaseURL.name;
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, _shareBaseURL)
        .then(
      (_) {
        if (shareBaseURLChanged) {
          FWEventBus.getInstance().fire(
            FWEvent(
              eventName: FWEventName.shareBaseURLUpdated,
            ),
          );
        }
      },
    );
  }

  String? _shareBaseURL;

  /// The configuration for ad badges.
  AdBadgeConfiguration? get adBadgeConfiguration => _adBadgeConfiguration;
  set adBadgeConfiguration(AdBadgeConfiguration? adBadgeConfiguration) {
    final valueHasChanged = _adBadgeConfiguration?.badgeTextType !=
            adBadgeConfiguration?.badgeTextType ||
        _adBadgeConfiguration?.backgroundColor !=
            adBadgeConfiguration?.backgroundColor ||
        _adBadgeConfiguration?.textColor != adBadgeConfiguration?.textColor;

    _adBadgeConfiguration = adBadgeConfiguration;

    final nativeMethodName =
        FireworkSDKNativeMethodName.setAdBadgeConfiguration.name;
    Map<String, dynamic> arg = {};
    if (adBadgeConfiguration != null) {
      arg = AdBadgeConfigurationUtil.toJson(adBadgeConfiguration);
    }
    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg)
        .then((_) {
      if (valueHasChanged) {
        FWEventBus.getInstance().fire(
          FWEvent(
            eventName: FWEventName.adBadgeConfigurationUpdated,
          ),
        );
      }
    });
  }

  AdBadgeConfiguration? _adBadgeConfiguration;

  /// Get video launch behavior.
  VideoLaunchBehavior? get videoLaunchBehavior => _videoLaunchBehavior;
  VideoLaunchBehavior? _videoLaunchBehavior;

  /// Get current app language.
  String? get appLanguage => _appLanguage;
  String? _appLanguage;

  bool get debugLogsEnabled => FWLoggerUtil.enabled;

  set debugLogsEnabled(bool value) {
    FWLoggerUtil.enabled = value;
  }

  static FireworkSDK _getInstance() {
    _instance ??= FireworkSDK._();
    return _instance!;
  }

  /// Get global single instance of FireworkSDK class.
  factory FireworkSDK.getInstance() => _getInstance();

  FireworkSDK._() {
    FWMethodChannelUtil.getFirewokSDKChannel()
        .setMethodCallHandler((call) async {
      final fwMethod = FireworkSDKMethodNameUtil.createMethodName(call.method);
      if (fwMethod == null) {
        return null;
      }

      switch (fwMethod) {
        case FireworkSDKMethodName.sdkInit:
          final sdkInitEvent = SDKInitEventUtil.fromJson(call.arguments);
          if (sdkInitEvent != null) {
            onSDKInit?.call(
              sdkInitEvent,
            );
          } else {
            onSDKInit?.call(
              SDKInitEvent(
                error: FWError(
                  name: FWErrorName.parseJsonError,
                  reason: "Fail to parse SDK init event",
                ),
              ),
            );
          }
          break;
        case FireworkSDKMethodName.customCTAClick:
          onCustomCTAClick?.call(
            CustomCTAClickEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.videoPlayback:
          onVideoPlayback?.call(
            VideoPlaybackEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.videoFeedClick:
          onVideoFeedClick?.call(
            VideoFeedClickEventUtil.fromJson(
              call.arguments,
            ),
          );
          break;
        case FireworkSDKMethodName.nativeAppLanguageUpdated:
          final arg = call.arguments;
          if (arg is Map<dynamic, dynamic>) {
            if (arg["appLanguage"] is String) {
              _updateInternalAppLanguage(arg["appLanguage"] as String);
            }
          }
          break;
      }

      return null;
    });
  }

  /// LiveStream instance.
  LiveStream get liveStream => LiveStream.getInstance();

  /// VideoShopping instance.
  VideoShopping get shopping => VideoShopping.getInstance();

  /// FWNavigator instance.
  FWNavigator get navigator => FWNavigator.getInstance();

  /// Initializes Firework SDK.
  ///
  /// [userId] is used to uniquely identify device or user.
  Future<void> init({
    String? userId,
    VideoLaunchBehavior? videoLaunchBehavior,
  }) async {
    final videoLaunchBehaviorUpdated =
        _videoLaunchBehavior != videoLaunchBehavior;

    final nativeMethodName = FireworkSDKNativeMethodName.init.name;
    Map<String, dynamic> arg = {};
    if (userId != null) {
      arg["userId"] = userId;
    }

    if (videoLaunchBehavior != null) {
      arg["videoLaunchBehavior"] = videoLaunchBehavior.name;
    }

    await FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg);
    if (videoLaunchBehaviorUpdated) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.videoLaunchBehaviorUpdated,
        ),
      );
    }

    final shoppingInitNativeMethodName = ShoppingNativeMethodName.init.name;
    await FWMethodChannelUtil.getShoppingChannel().invokeMethod(
      shoppingInitNativeMethodName,
    );

    final liveStreamInitNativeMethodName = LiveStreamNativeMethodName.init.name;
    await FWMethodChannelUtil.getLiveStreamMethodChannel()
        .invokeMethod(liveStreamInitNativeMethodName);

    FWGlobalState.getInstance().completeSdkInitCalledFuture();
  }

  /// Opens video [url] with the [config].
  Future<void> openVideoPlayer({
    required String url,
    OpenVideoPlayerConfiguration? config,
  }) async {
    if (!FWGlobalState.getInstance().sdkInitCalled) {
      await FWGlobalState.getInstance().sdkInitCalledFuture;
    }
    final nativeMethodName = FireworkSDKNativeMethodName.openVideoPlayer.name;
    Map<String, dynamic> arg = {
      "url": url,
    };

    if (config != null) {
      arg["config"] = VideoPlayerConfigurationUtil.toJson(config);
    }

    FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, arg);
  }

  /// Records the user purchase.
  Future<void> trackPurchase(TrackPurchaseParameters parameters) async {
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      if (!FWGlobalState.getInstance().sdkInitCalled) {
        await FWGlobalState.getInstance().sdkInitCalledFuture;
      }
      final nativeMethodName = FireworkSDKNativeMethodName.trackPurchase.name;
      FWMethodChannelUtil.getFirewokSDKChannel().invokeMethod(
          nativeMethodName, TrackPurchaseParametersUtil.toJson(parameters));
    }
  }

  /// Change App level language.
  /// language can be en, ar and en-US.
  /// If language is null or empty string, SDK will use system language.
  Future<bool> changeAppLanguage(String? language) async {
    if (!FWGlobalState.getInstance().sdkInitCalled) {
      await FWGlobalState.getInstance().sdkInitCalledFuture;
    }

    final nativeMethodName = FireworkSDKNativeMethodName.changeAppLanguage.name;
    final result = await FWMethodChannelUtil.getFirewokSDKChannel()
        .invokeMethod(nativeMethodName, language);
    if (result) {
      if (defaultTargetPlatform == TargetPlatform.iOS) {
        await navigator.stopFloatingPlayer();
      }
      _updateInternalAppLanguage(language);
    }
    return result;
  }

  void _updateInternalAppLanguage(String? language) {
    final valueHasChanged = language != _appLanguage;
    _appLanguage = language;
    if (valueHasChanged) {
      FWEventBus.getInstance().fire(
        FWEvent(
          eventName: FWEventName.appLanguageUpdated,
        ),
      );
    }
  }
}
